@@ -1,6 +1,7 @@
 # frozen_string_literal: true
 require 'rubygems/test_case'
 require 'rubygems'
+require 'shellwords'
 
 class TestConfig < Gem::TestCase
 
@@ -8,17 +9,21 @@ class TestConfig < Gem::TestCase
     util_make_gems
     spec = Gem::Specification.find_by_name("a")
     spec.activate
-    assert_equal "#{spec.full_gem_path}/data/a", Gem.datadir('a')
+    assert_equal "#{spec.full_gem_path}/data/a", spec.datadir
   end
 
   def test_good_rake_path_is_escaped
     path = Gem::TestCase.class_eval('@@good_rake')
-    assert_match(/#{Gem.ruby} "[^"]*good_rake.rb"/, path)
+    ruby, rake = path.shellsplit
+    assert_equal(Gem.ruby, ruby)
+    assert_match(/\/good_rake.rb\z/, rake)
   end
 
   def test_bad_rake_path_is_escaped
     path = Gem::TestCase.class_eval('@@bad_rake')
-    assert_match(/#{Gem.ruby} "[^"]*bad_rake.rb"/, path)
+    ruby, rake = path.shellsplit
+    assert_equal(Gem.ruby, ruby)
+    assert_match(/\/bad_rake.rb\z/, rake)
   end
 
 end
